---
title: "Formatumwandlung long wide Format"
output:
  html_document:
    highlight: tango
    toc: true
params:
  soln: TRUE
  todo: FALSE
---

### [Rmd](long_wide.Rmd)

`r if(!params$soln) {"\\begin{comment}"}`
Lösungen mit angegeben
`r if(!params$soln) {"\\end{comment}"}`


## Long wide Umformatierung von Datensätzen mit library(tidyr)

Paket von Hadley Wickham. Teil von [tidyverse](https://github.com/tidyverse/tidyr)

Ziel sind tidy data

tidyr [documentation](https://www.rdocumentation.org/packages/tidyr/versions/0.6.1)

Befehle:

```
tidyr::gather()     # wide -> long
tidyr::spread()     # long -> wide
tidyr::separate()   # seperate one column in multiple columns (generating adequate factors)
tidyr::extract()    # 
tidyr::unite()      # Unite multiple columns into one, complement to separate()

vignette("tidy-data")
demo(package = "tidyr")

```


[tutorial](http://jules32.github.io/2016-07-12-Oxford/dplyr_tidyr/)


### Verstehbeispiel

subj:   Versuchsperson
gender: 'f', 'm'
t1:     AV Zeitpunkt 1
t2:     AV Zeitpunkt 2

```{r 1}
# wide format
df <- data.frame(
  subj   = c(1,2,3,4),
  gender = c('f', 'f', 'm', 'm'),
  t1     = c(4, 5, 7, 6),
  t2     = c(6, 5, 9, 8)
  )

require(tidyr)
df.t <- df %>% tidyr::gather(key=time, value=dv, t1, t2)
# or even shorter: key and value can be used as positional parameters too
df.t <- df %>% tidyr::gather(time, dv, t1, t2)

# we want to have all repeated measures for each person in consecutive lines
df.t %>% dplyr::arrange(subj, time) -> df.t
# we notice that variables subj and gender are kept and adequately repeated
df.t
```



### Verstehbeispiel 2*3 reines Messwiederholungsdesign

AV Zahl reproduzierter Silben
subj:   Versuchsperson
gender: 'f', 'm'
mnemotechnik: 'm', 'c'
t1:     AV Zeitpunkt 1
t2:     AV Zeitpunkt 2
t3:     AV Zeitpunkt 3

Die Idee hinter dem Umbau von wide nach long ist hier: 
- wandle im ersten Schritt __alle__ Messwiederholungswerte in eine Spalte um. 
- Die Information, aus welcher Spalte die Werte stammen, stehen dann in der neu generierten Spalte mit Namen des Paramter 'key' bzw. 
  des zweiten Parameters des gather-Befehls.
- in einem weiteren Befehl werden die Infos aus der neu generierten key-Spalte in neue Spalten aufgesplittet, 
  die die jeweils relevanten Informationen für den Messwiederholungsfaktor enthalten. 
  Die Key-Spalte wird quasi entschlüsselt.
  
```{r 2}
# we generate data frame in wide format
df <- data.frame(
  subj   = c(1,2,3,4),
  gender = c('f', 'f', 'm', 'm'),
  t1c     = c( 4,  5,  7,  6),
  t2c     = c( 6,  5,  9,  8),
  t3c     = c( 7,  6, 12, 10),
  t1m     = c( 8,  6, 15, 10),
  t2m     = c(12,  7, 19, 13),
  t3m     = c(13,  7, 22, 14)
  )
# take a look at it
df

# we gather all repeated measure variables at once
require(tidyr)
df.t <- df %>% tidyr::gather(key=m_t, value=dv, t1c:t3m)
# now we have all information mixed in column m_t
head(df.t)
# we make new columns to specify the repeated measure factors using the position in source column name and preserving source column to control what's happening
df.t %>% tidyr::separate(m_t, c("time", "cond"), 2, 1, remove=F) -> df.t  # 2, 1 refer to the number of letters in value strings
df.t %>% tidyr::separate(m_t, c("time", "cond"), 2, -1, remove=F) -> df.t # -1 refers to the last letter in value string
# more separation techniques are  possible
head(df.t)
# we want to have all repeated measures for each person in consecutive lines and reorder the data frame
head(df.t)
```





## Long wide Umformatierung von Datensätzen mit library(reshape2)

Befehle in diesem Kontext

`stack()` und `unstack()`

`library(reshape2)`
`melt()` wide -> long format
`cast()` long -> wide (und noch mehr)

`library(reshape2)` und `library(reshape)` haben namespace Probleme, beispielsweise bei 'variable.name', 'value.name' und 'variable_name'.
Um reshape2 sauber zu verwenden, muss reshape entladen sein `detach(package:reshape)`.


### Verstehbeispiel

subj:   Versuchsperson
gender: 'f', 'm'
t1:     AV Zeitpunkt 1
t2:     AV Zeitpunkt 2

```{r 3}
# wide format
dd <- data.frame(
  subj   = c(1,2,3,4),
  gender = c('f', 'f', 'm', 'm'),
  t1     = c(4, 5, 7, 6),
  t2     = c(6, 5, 9, 8)
  )


# we could also have used reshape2
require("reshape2")
reshape2::melt(dd, id=c('subj', 'gender'), measured=c('t1', 't2'))

# more verbatim
dd.t <- reshape2::melt(dd, id=c('subj', 'gender'), variable.name="time", value.name="dv", measured=c('t1', 't2'))
dd.t


# the same using tidyr of tidyverse
# cf vignette("tidy-data")
require(tidyr)
# the values of the two columns t1 and t2 are gathered in one column called dv
# a new column "time" is created, where t1 and t2 are coded (factor levels)
dd %>% tidyr::gather(time, dv, t1, t2) -> dd.l
dd.l

# change to wide format
# factor levels of column time become column names
# dv values are put to columns, which names are equal to factor levels of column time
dd.l %>% tidyr::spread(time, dv)

```


## Long wide Umformatierung von Datensätzen mit base system bzw. Slicing

Grundidee: Datentabelle in Teiltabellen aufteilen, Info-Spalte hinzufügen und Teiltabellen wieder zusammenfügen.

```{r base}
# wide format
dd <- data.frame(
  subj   = c(1,2,3,4),
  gender = c('f', 'f', 'm', 'm'),
  t1     = c(4, 5, 7, 6),
  t2     = c(6, 5, 9, 8)
  )

# we create a dataframe of t1 data
# dd.t1 <- dd[1:3] # or more verbose
dd.t1 <- dd[, c('subj', 'gender', 't1')]
# we rename column t1
dd.t1['y'] <- dd.t1$t1
dd.t1$t1 <- NULL
# and add t1 info
dd.t1['time'] <- 't1'
# the same wit t2 data
dd.t2 <- dd[, c('subj', 'gender', 't2')]
# and add t1 info
dd.t2['time'] <- 't2'
# we rename column t2
dd.t2['y'] <- dd.t2$t2
dd.t2$t2 <- NULL

# we now merge the two parts
dd.t <- rbind(dd.t1, dd.t2)

```



# Beispiele

### Beispiel STAI

Stai State sei erhoben worden von männlichen und weiblichen Studierenden.

Erhoben wurde in 3 Situationen: 

- neutral (vor einem Statistik-Tutorium)
- exam1 (retrospektiv nach einer ersten Statistik-Klausur)
- exam2 (retrospektiv nach einer zweiten Statistik-Klausur)

Die Studierenden hatten in ihrem Abitur eine niedrige (lo), eine durchschnittliche (av) oder eine gute (hi) Note in Mathematik.


```{r}
require("reshape2")
require(psych)
# minimal example
ddf <- read.delim("http://md.psych.bio.uni-goettingen.de/data/virt/v_stait_exam_wide.txt")
# what is in there
head(ddf)
psych::describe(ddf)
```

Die Daten spiegeln eine Struktur, wie sie von vielen Statistikpaketen verlangt wird.
Pro Zeile eine Versuchsperson (Beobachtungseinheit).
Alle Daten dieser Zeile sind Daten dieser Beobachtungseinheit.
Hieraus folgt, dass 'neutral', 'exam1' und 'exam2' Messwiederholungsdaten sein müssen.


R verlangt bei Messwiederholungen das 'long format'.
Eine Beobachtung im gegebenen Beispiel ist eine Kombination aus Zeitpunkt und Versuchsperson.
Hier werden auch die Messwiederholungsvariablen im Data-Frame so angeordnet, als seien es Beobachtungen (im klassischen Sinn).
Die Unterscheidung von unabhängigen und abhängigen Variablen (between subjects und within subjects) erfolgt über die Formulae beim Aufruf der entsprechenden Auswertungsmethoden.
Im Long-Format werden auch die Messwiederholungsdaten über eigene Variablen kodiert, nicht nur die Zugehörigkeit der Beobachtungen zu bestimmten (Sub-)Gruppen.
Die zu einer Beobachtung gehörenden 'Gruppierungsvariablen' werden im Long-Format entsprechend wiederholt.


`r if(!params$soln) {"\\begin{comment}"}`

#### Ansatz mit tidyr


```{r}
ddf <- read.delim("http://md.psych.bio.uni-goettingen.de/data/virt/v_stait_exam_wide.txt")
require("tidyr")

# convert it to long format 
ddf.r <- ddf %>% tidyr::gather(key=situation, value=dv, neutr, exam1, exam2)
# show it
head(ddf.r)

# we can even leave off "key=..." and "value=..." and use positional arguments only 
# ddf %>% tidyr::gather(situation, dv, neutr, exam1, exam2)
```


#### Ansatz mit reshape2

```{r}
require("reshape2")
ddf <- read.delim("http://md.psych.bio.uni-goettingen.de/data/virt/v_stait_exam_wide.txt")
# convert it to long format 
ddf.r <- melt(ddf, id=c('subj','gender', 'group'), variable.name="sit", value.name="anx", measured=c('neutr', 'exam1', 'exam2'))
# show it
head(ddf.r)
# sequence of id variables matters for the sequence of columns in the newly created data frame
head(melt(ddf, id=c('subj', 'group', 'gender'), variable.name="sit", value.name="anx", measured=c('exam1', 'exam2', 'neutr')))

```


#### Ansatz mit reshape2

Sollte es notwendig sein, die Daten aus dem Long-Format in das Wide-Format zurückzuwandeln, kann der Befehl `dcast()` (für Data-Frames) aus der `library(reshape2)` verwendet werden.

```{r}
require("reshape2")
ddf <- read.delim("http://md.psych.bio.uni-goettingen.de/data/virt/v_stait_exam_wide.txt")
# convert it to long format 
ddf.r <- melt(ddf, id=c('subj','gender', 'group'), measured=c('neutr', 'exam1', 'exam2'))

# convert long format back to wide format 
head(dcast(ddf.r, subj + gender + group ~ variable, value ="value"))
# compare original data
head(ddf)
```

`r if(!params$soln) {"\\end{comment}"}`



`r if(!params$todo) {"\\begin{comment}"}`


### Beispiel: Sets von Variablen pro Messzeitpunkt


```{r}
df <- data.frame(
  id = 1:10,
  time = as.Date('2009-01-01') + 0:9,
  Q1_1 = rnorm(10, 0, 1),
  Q1_2 = rnorm(10, 0, 1),
  Q1_3 = rnorm(10, 0, 1),
  Q2_1 = rnorm(10, 0, 1),
  Q2_2 = rnorm(10, 0, 1),
  Q2_3 = rnorm(10, 0, 1)
)
df
df %>%
  gather(key, value, -id, -time) %>%
  extract(key, c("question", "loop_number"), "(Q.)_(.)") %>%
  spread(question, value)


```





### Gantt Data

In diesem Beispiel geht es einerseits um das Einlesen von Json-Daten (ein spezielles Austauschformat) und andererseits um die Aufbereitung einer "langen Zeile".
Es wird nur eine einzige Beobachtung eingelesen und von wide nach long konvertiert.

```{r}
EHC = as.data.frame(jsonlite::fromJSON("{\"current_address_M1\":[\"\"],\"current_address_M2\":[\"\"],\"current_address_M3\":[1],\"current_address_M4\":[null],\"current_address_M5\":[null],\"current_address_M6\":[null],\"current_address_M7\":[null],\"current_address_M8\":[null],\"current_address_M9\":[null],\"current_address_M10\":[null],\"current_job_J1\":[\"\"],\"current_job_J2\":[null],\"current_job_J3\":[null],\"current_job_J4\":[null],\"current_job_J5\":[null],\"current_job_J6\":[null],\"current_job_J7\":[null],\"current_job_J8\":[null],\"current_job_J9\":[null],\"current_job_J10\":[null],\"datefrom_M1\":[\"2010-01-01\"],\"dateto_M1\":[\"2011-12-01\"],\"datefrom_M2\":[\"2012-01-01\"],\"dateto_M2\":[\"2012-12-01\"],\"datefrom_M3\":[\"2013-01-01\"],\"dateto_M3\":[null],\"datefrom_M4\":[null],\"dateto_M4\":[null],\"datefrom_M5\":[null],\"dateto_M5\":[null],\"datefrom_M6\":[null],\"dateto_M6\":[null],\"datefrom_M7\":[null],\"dateto_M7\":[null],\"datefrom_M8\":[null],\"dateto_M8\":[null],\"datefrom_M9\":[null],\"dateto_M9\":[null],\"datefrom_M10\":[null],\"dateto_M10\":[null],\"datefrom_J1\":[\"2014-01-01\"],\"dateto_J1\":[\"2016-12-01\"],\"datefrom_J2\":[null],\"dateto_J2\":[null],\"datefrom_J3\":[null],\"dateto_J3\":[null],\"datefrom_J4\":[null],\"dateto_J4\":[null],\"datefrom_J5\":[null],\"dateto_J5\":[null],\"datefrom_J6\":[null],\"dateto_J6\":[null],\"datefrom_J7\":[null],\"dateto_J7\":[null],\"datefrom_J8\":[null],\"dateto_J8\":[null],\"datefrom_J9\":[null],\"dateto_J9\":[null],\"datefrom_J10\":[null],\"dateto_J10\":[null],\"datefrom_E1\":[\"2014-02-01\"],\"datefrom_E2\":[\"2015-11-01\"],\"datefrom_E3\":[\"2017-11-01\"],\"datefrom_E4\":[null],\"datefrom_E5\":[null],\"datefrom_E6\":[null],\"datefrom_E7\":[null],\"datefrom_E8\":[null],\"datefrom_E9\":[null],\"datefrom_E10\":[null],\"count_SN\":[3],\"count_SJ\":[1],\"count_SE\":[3],\"address_M1\":[\"Gosslerstr. 14\"],\"address_M2\":[\"Weende\"],\"address_M3\":[\"Goettingen, Germany\"],\"address_M4\":[null],\"address_M5\":[null],\"address_M6\":[null],\"address_M7\":[null],\"address_M8\":[null],\"address_M9\":[null],\"address_M10\":[null],\"jobtitle_J1\":[\"Web Developer\"],\"jobtitle_J2\":[null],\"jobtitle_J3\":[null],\"jobtitle_J4\":[null],\"jobtitle_J5\":[null],\"jobtitle_J6\":[null],\"jobtitle_J7\":[null],\"jobtitle_J8\":[null],\"jobtitle_J9\":[null],\"jobtitle_J10\":[null],\"eventtitle_E1\":[\"Engagement\"],\"eventtitle_E2\":[\"Marriage\"],\"eventtitle_E3\":[\"Baby\"],\"eventtitle_E4\":[null],\"eventtitle_E5\":[null],\"eventtitle_E6\":[null],\"eventtitle_E7\":[null],\"eventtitle_E8\":[null],\"eventtitle_E9\":[null],\"eventtitle_E10\":[null]}"), stringsAsFactors=F)


address_cols <- grep('^address_M', colnames(EHC));
addresses <- EHC[, address_cols]
# addresses <- addresses[, colSums(is.na(addresses)) == 0];


curr_address_cols <- grep('^current_address_M', colnames(EHC));
is_current <- unlist(EHC[, curr_address_cols])
# is_current <- is_current[, colSums(is.na(is_current)) == 0];

fromdates_cols <- grep('^datefrom_M', colnames(EHC));
fromdates <- unlist(EHC[, fromdates_cols])
# fromdates <- fromdates[, colSums(is.na(fromdates)) == 0];

todates_cols <- grep('^dateto_M', colnames(EHC));
todates <- unlist(EHC[, todates_cols])
# todates <- todates[, colSums(is.na(todates)) == 0];

data <-cbind(address = addresses, froms = fromdates, tos = todates, current = is_current)

dd <- EHC[,grep('_M', names(EHC))]
dd
#dd <- as.character(dd)

require(dplyr)
# we need tidyr, a part of tidyverse for long -> wide -> long conversion
require(tidyr)
# or even require(tidyverse)

dd %>%
  tidyr::gather(key, value, grep('date', names(dd))) %>%
  tidyr::separate(key, c("data", "time"), sep="_M", remove=F) %>%
  tidyr::spread(data, value) %>%
  dplyr::select(grep("^address_M", names(.)), datefrom, dateto)

```




### multiple ...

[source](https://stackoverflow.com/questions/29146287/long-to-wide-data-with-tidyr)

```{r}
df.l = data.frame(name=c("A","A","B","B"),
                group=c("g1","g2","g1","g2"),
                V1=c(10,40,20,30),
                V2=c(6,3,1,7))
# should be converted to
df.w = data.frame(name=c("A", "B"),               
                V1.g1=c(10,20),
                V1.g2=c(40,30),
                V2.g1=c(6,1),
                V2.g2=c(3,7))
# using reshape in base package
reshape(df.l, idvar='name', timevar='group', direction='wide')

# using ...
require(data.table) # v1.9.5+
# dcast(setDT(df.l), name ~ group, value.var=c("V1", "V2"))

```

`r if(!params$soln) {"\\end{comment}"}`


## Beispiele Übungen



### Beispieldaten Reaktionszeiten

inspiriert von einer [Reaktionszeitstudie](https://www.ncbi.nlm.nih.gov/pmc/articles/PMC2586814/)

- der unten angelegte Datensatz ist im Wide-Format
- überlegen Sie eine geeignete Struktur der Daten im Long-Format
- transferieren Sie die Daten enstprechend in einen neuen DataFrame


#### Infos

- Schwierigkeit: single task < mixed non switch < mixed switch 
- congruent (green: go, red: stop) < incongruent (green: stop, red: go)
- je tiefer (weiter rechts), desto schwieriger

```{r}
# reaction times dataframe, wide format
df.w <- data.frame(
	subj   = c(    1,    2,    3,    4,    5,    6,    7,    8,    9,  10,   11,   12),
	gender = c(rep(c(rep("f", 2), rep("m", 2)), 3)),
	age    = c(round(runif(12, 18, 40))),
	#age    = c(  18, 35)
	group  = c(rep(1:3, each=4)),  
	#            fy    fy    my    my    fm    fm    mm    mm    fo    fo    mo    mo
	ss.c   = c( 785,  780,  770,  775,  855,  862,  842,  838,  934,  932,  944,  946),  # single task, congruent
	ss.i   = c( 888,  892,  886,  894,  957,  963,  958,  965, 1039, 1042, 1037, 1045),  # single task, incongruent
	mn.c   = c( 884,  886,  873,  876,  949,  950,  939,  941, 1042, 1045, 1025, 1027),  # mixed, non switch, congruent
	mn.i   = c( 962,  958,  939,  942, 1019, 1021, 1001,  998, 1121, 1118, 1111, 1108),  # mixed, non switch, incongruent
	ms.c   = c(1054, 1054, 1006, 1007, 1157, 1162, 1108, 1112, 1314, 1317, 1268, 1262),  # mixed, switch, congruent
	ms.i   = c(1202, 1197, 1103, 1096, 1354, 1346, 1265, 1255, 1357, 1533, 1446, 1444)   # mixed, switch, incongruent
	)

```



http://ac.els-cdn.com/0013469480902205/1-s2.0-0013469480902205-main.pdf
https://www.ncbi.nlm.nih.gov/pmc/articles/PMC3031137/






### Ein komplexerer Datensatz von Reaktionszeit- und EEG-Daten


#### Hintergrund


ispired by:

[age differences in reaction time](https://www.researchgate.net/publication/15185644_Age_Differences_and_Changes_in_Reaction_Time_The_Baltimore_Longitudinal_Study_of_Aging)
[local copy](papers_rt_p300/Age_Differences_and_Changes_in_Reaction_Time_The_B.pdf)

[age differences in p300](https://www.researchgate.net/publication/222464083_Age_differences_in_P3-reaction_time_associations)
[local copy](papers_rt_p300/age_differences_p300_rt_associations.pdf)

auditory and visual P300 [local copy](p300_age_auditory_visual.pdf)

task complexity and P300[local copy](p300_task_complexity_smulders_ea.pdf)


- Signalton: 1000 Hz oder 250 Hz Ton, 300 ms Länge
- Reaktion auf "hand held response button"
- 6-13 s Inter Stimulus Interval
- Simple reaction time: Reaktion auf beide Signaltöne
- discriminative reaction time: Reaktion nur auf einen der beiden Signaltöne (go-nogo-paradigm)
- P300 at PZ

- P300 amplitude was larger and latency longer for the visual compared with the auditory stimulus
- P300 latency ist höher für schwierigere Aufgaben (z.B. Wahrnehmungsdiskrimination)

```{r}
# reaction times dataframe, wide format
df.w <- data.frame(
	subj     = c(    1,    2,    3,    4,    5,    6,    7,    8,    9,   10,   11,   12),
	gender   = c(rep(c(rep("f", 2), rep("m", 2)), 3)),
	age      = c(   17,   22,   18,   21,   46,   53,   48,   56,   75,   85,   76,   83),
	age.gr   = c(rep(c("15-25", "45-55", "75-85"), each=4)),  
	#               fy    fy    my    my    fm    fm    mm    mm    fo    fo    mo    mo
  # RT 
	a.rt.sr = c(  235,  230,  219,  215,  270,  266,  254,  250,  283,  279,  268,  262),  # auditory, simple reaction time
	a.rt.dr = c(  403,  396,  358,  352,  444,  437,  388,  381,  492,  485,  434,  427),  # auditory, disjuncitive reaction time
	v.rt.sr = c(  275,  273,  262,  254,  307,  307,  296,  290,  323,  321,  305,  303),  # visual, simple reaction time
	v.rt.dr = c(  446,  437,  397,  390,  484,  479,  425,  418,  530,  525,  478,  469),  # visual, disjuncitive reaction time
  # p300 amplitude
	a.p3a.sr = c( 7.40, 7.21, 6.16, 7.57, 5.60, 3.68, 4.21, 3.89, 0.78, 1.78, 2.86, 1.79),  # auditory, p300 amplitude, simple RT
	a.p3a.dr = c( 6.64, 6.35, 5.79, 7.02, 4.86, 3.27, 3.61, 3.17, 0.28, 1.29, 2.52, 1.42),  # auditory, p300 amplitude, disjuncitive RT
	v.p3a.sr = c(10.47,10.04, 8.96,10.21, 8.34, 6.80, 6.71, 6.72, 3.69, 4.27, 5.58, 4.59),  # visual, p300 amplitude, simple RT
  v.p3a.dr = c( 9.92, 9.62, 8.36, 9.59, 7.75, 6.05, 6.15, 6.04, 3.22, 3.70, 5.08, 3.81),  # visual, p300 amplitude, disjuncitive RT
  # p300 latency
	a.p3l.sr = c(  385,  394,  382,  399,  440,  449,  439,  452,  495,  513,  498,  508),  # auditory, p300 latency, simple RT
  a.p3l.dr = c(  420,  437,  417,  439,  502,  505,  495,  509,  559,  581,  569,  582),  # auditory, p300 latency, disjuncitive RT
  v.p3l.sr = c(  422,  430,  420,  440,  476,  484,  472,  483,  535,  545,  531,  544),  # visual, p300 latency, simple RT
  v.p3l.dr = c(  459,  465,  452,  473,  534,  537,  527,  541,  602,  622,  608,  619)   # visual, p300 latency, disjuncitive RT
	)
getwd()
# write.table(df.w, "rt_p3_w.txt", quote=F, sep="\t", row.names=F)
```

Konvertiere obigen DataFrame in ein geeignetes Long-Format

Beachte: Was sind verschiedene Variablen und was ist eine Messwiederholung derselben Variable?

`r if(!params$soln) {"\\begin{comment}"}`
[res_begin:rt_p3]

Hier ein Ansatz, der die reactin time, die P300 Amplitude und die P300 Latency jeweils als eigene Reaktionsvariable auffasst 
und den Long-Dataframe entsprechend organisiert.

```{r}
# we convert data to wide format
require(dplyr)
require(tidyr)
# we start with writing 3 data frames for the 3 types of dv (RT, p300 amplitude, p300 latency)
df.w %>% dplyr::select(subj:v.rt.dr) -> df.rt
df.w %>% dplyr::select(subj, a.p3a.sr:v.p3a.dr) -> df.p3a
df.w %>% dplyr::select(subj, a.p3l.sr:v.p3l.dr) -> df.p3l
# we convert each of them to long format
df.rt  %>% tidyr::gather(key,  rt, a.rt.sr:v.rt.dr)    %>% 
  tidyr::extract(key, c("av", "complex"), "([a,v])\\..+\\.(.+)", remove=F) %>%
  tidyr::unite(subj_key, subj, av, complex, sep="_", remove=F) %>% 
  # dplyr::select(-subj) %>%
  dplyr::arrange(subj_key)  -> df.rt

df.p3a %>% tidyr::gather(key.a, p3a, a.p3a.sr:v.p3a.dr) %>% 
  tidyr::extract(key.a, c("av.a", "complex.a"), "([a,v])\\..+\\.(.+)", remove=F) %>%
  tidyr::unite(subj_key, subj, av.a, complex.a, sep="_", remove=F) %>% 
  dplyr::select(-subj) %>%
  dplyr::arrange(subj_key)  -> df.p3a

df.p3l %>% tidyr::gather(key.l, p3l, a.p3l.sr:v.p3l.dr) %>% 
  tidyr::extract(key.l, c("av.l", "complex.l"), "([a,v])\\..+\\.(.+)", remove=F) %>%
  tidyr::unite(subj_key, subj, av.l, complex.l, sep="_", remove=F) %>% 
  dplyr::select(-subj) %>%
  dplyr::arrange(subj_key)  -> df.p3l

  
#   
#     dplyr::arrange(subj, key.p3a) -> df.p3a
# df.p3l %>% tidyr::gather(key.p3l, value.p3l, a.p3l.sr:v.p3l.dr) %>% dplyr::arrange(subj, key.p3l) -> df.p3l

# we join the long format dataframes to one dataframe
df.t <- df.rt %>%  dplyr::left_join(df.p3a, by="subj_key") %>%  dplyr::left_join(df.p3l, by="subj_key") %>% dplyr::arrange(subj, subj_key)

# after controlling we don't need the redundant variables any more
df.t %>% dplyr::select(-key, -key.a, -av.a, -complex.a, -key.l, -av.l, -complex.l) -> df.t

# write.table(df.t, "rt_p3.txt", quote=F, sep="\t", row.names=F)

# data are available under
# df.w <- read.delim("http://md.psych.bio.uni-goettingen.de/data/virt/rt_p3_w.txt")
# df <- read.delim("http://md.psych.bio.uni-goettingen.de/data/virt/rt_p3.txt")

```

[res_end:rt_p3]
`r if(!params$soln) {"\\end{comment}"}`



`r if(!params$todo) {"\\begin{comment}"}`
#### Exam Data

[Exam Data](http://md.psych.bio.uni-goettingen.de/data/virt/v_stait_exam_mixed.txt)

```{r}
require(tidyverse)
dd <- read_delim("http://md.psych.bio.uni-goettingen.de/data/virt/v_stait_exam_mixed.txt", delim='\t')
```


Die Daten sind "durcheinander".
Die Spaltennamen codieren die Messung.
Die Personen waren entweder weiblich `_f` oder männlich `_m`.
Alle Personen durchliefen die Situation n, e1, e2 (neutral, exam 1, exam 2). 
Dies ist codiert mit `_n_`, `_e1_`, `_e2_`.
Die Personen hatten in ihrem Abitur eine niedrige (`lo_`), eine durchschnittliche (`av_`) oder eine gute (`hi_`) Note in Mathematik.


Führen Sie die obigen Daten in das für viele Statistikpaket übliche Format über, 
in dem alle Messwiederholungsvariablen in einer Zeile pro Beobachtung sind.

Führen Sie obige Daten in das Long-Format über.


`r if(!params$todo) {"\\end{comment}"}`

#### Studierenden STAI Daten

Erstellen Sie ein Dataframe im Long-Format, mit dem Sie die Messzeitpunkte 1 und 2 hinsichtlich der STAI-Skalen vergleichen können.
[Daten](http://md.psych.bio.uni-goettingen.de/data/div/stud_stai_scales_utf8.txt)

- alle Variablennamen, die mit einer '2' enden, stammen vom Messzeitpunkt 2
- bei den state- und trait-Werten kommt danach noch '_s' oder '_m'. 
  Dies sind die aggregierten Stai-Skalenwerte (state oder trait), entweder als Summe über die Itemwerte gebildet ('_s') 
  oder als Mittelwert über die Itemwerte ('_m')


`r if(!params$soln) {"\\begin{comment}"}`
__Lösung__

```{r}
require(tidyverse)
dd <- read_delim("http://md.psych.bio.uni-goettingen.de/data/div/stud_stai_scales_utf8.txt", delim='\t')



```

`r if(!params$soln) {"\\end{comment}"}`


### Links und Referenzen

[Reshaping Your Data with tidyr](https://uc-r.github.io/tidyr)

Tidyr [reference](http://tidyr.tidyverse.org/reference/)

Reorganizing the data for within subject analyses
in [Link](http://www.personality-project.org/r/r.guide/r.anova.html)
<!-- [Link](http://www.personality-project.org/r/r.anova.html) -->

Sean C. Anderson (2013) [An Introduction to reshape2](http://seananderson.ca/2013/10/19/reshape.html)

https://stackoverflow.com/questions/5890584/how-to-reshape-data-from-long-to-wide-format

https://stackoverflow.com/questions/9341865/alternatives-to-statsreshape

http://www.milanor.net/blog/reshape-data-r-tidyr-vs-reshape2/
